# Vtex Auth
APP_KEY_HEADER = "X-VTEX-API-AppKey"
APP_TOKEN_HEADER = "X-VTEX-API-AppToken"  # noqa: S105

# Request Defaults
DEFAULT_TIMEOUT = 10
DEFAULT_RETRIES = 5
DEFAULT_RAISE_FOR_STATUS = False

# Retry
RETRIABLE_STATUSES = {
    401, 407, 408, 409, 425, 429, 500, 502, 503, 504, 507, 511, 525,
}

# Generics
MIN_PAGE_SIZE = 1

# Logistics
LIST_CARRIERS_START_PAGE = 1
LIST_CARRIERS_MAX_PAGE_SIZE = 1000

LIST_DOCKS_START_PAGE = 1
LIST_DOCKS_MAX_PAGE_SIZE = 1000

LIST_SHIPPING_POLICIES_START_PAGE = 1
LIST_SHIPPING_POLICIES_MAX_PAGE_SIZE = 1000

# Orders
LIST_FEED_ORDERS_MAX_PAGE_SIZE = 10
LIST_ORDERS_START_PAGE = 1
LIST_ORDERS_MAX_PAGE = 30
LIST_ORDERS_MAX_PAGE_SIZE = 100

# Transactions
LIST_TRANSACTION_INTERACTIONS_RETRIES = 10
